#include <raylib.h>
#include "preferred_size.h"
#include <math.h>

int main()
{
  int screenWidth, screenHeight;
  GetPreferredSize(&screenWidth, &screenHeight);
  InitWindow(screenWidth, screenHeight, "Billboard rendering");
  SetTargetFPS(30);

  // Define the camera to look into our 3d world
  Camera camera = {0};
  camera.position = (Vector3){5.0f, 8.0f, 5.0f}; // Camera position
  camera.target = (Vector3){0.0f, 0.5f, 0.0f};   // Camera looking at point
  camera.up = (Vector3){0.0f, 1.0f, 0.0f};       // Camera up vector (rotation towards target)
  camera.fovy = 45.0f;                           // Camera field-of-view Y
  camera.projection = CAMERA_PERSPECTIVE;        // Camera projection type

  Texture2D bill = LoadTexture("data/sprite.png");
  Vector3 billPositionStatic = {0.0f, 2.0f, 0.0f};

  while (!WindowShouldClose())
  {
    if (IsPaused())
    {
      // canvas is not visible in browser - do nothing
      continue;
    }
    BeginDrawing();
    ClearBackground(DARKGREEN);
    camera.position.x = sinf(GetTime() * 0.25f) * 5.0f;
    if (IsMouseButtonDown(MOUSE_LEFT_BUTTON))
    {
      camera.position.y += GetMouseDelta().y * 0.05f;
    }
    
    BeginMode3D(camera);
    DrawGrid(4, 1.0f);
    DrawCube((Vector3){0.0f, 0.0f, 0.0f}, 1.0f, 1.0f, 1.0f, BLUE);
    for (float z = -10.0f; z <= 10.0f; z += 2.0f)
    {
      Vector3 billPosition = billPositionStatic;
      billPosition.z = z;
      // a little hopping
      billPosition.y = fabsf(sinf(GetTime() * 5.0f + z) * 0.25f);
      DrawBillboard(camera, bill, billPosition, 2.0f, WHITE);
    }
    EndMode3D();
    EndDrawing();
  }

  CloseWindow();

  return 0;
}